/*
		Interfaz basica para consola - WBFS
		
		Programa creado por: Ignacio Dominguez - 4ltrv1sm0
		
		La modificacion, copia y/o distribucion de este programa es libre. El autor no se hace responsable
		de los daos o problemas que pueda ocasionar. Usa este software bajo tu propia responsabilidad.
		
		El funcionamiento de este software requiere el archivo wbfs ejecutable en el mismo directorio.

*/

#include <stdio.h>												// includes necesarios
#include <string.h>
#include <unistd.h>

#define		MAX_RUTA	256										// definiciones, MAX_RUTA se utiliza tanto para nombres de archivo como para rutas con
#define		VERSION		0.11									// nombres de archivo
#define		ADD			1										// opciones del menu
#define		LIST		2
#define		DEL			3
#define		EXTRACT		4
#define		MAKEHBC		5
#define		FORMAT		6
#define		CAMBIAR		7
#define		SALIR		0

int menu(const char disk_id[])									// Muestra el menu, recoge la opcion introducida y la devuelve
{
	int opcion = -1;
	
	while (opcion > 7 || opcion < 0)
	{
		system("clear");										// limpia la pantalla
																// muestra el texto del menu
		printf("-------------------------------\n WBFS easy-console - 4ltrv1sm0\n Licencia GPL      - ver %.2f\n-------------------------------\n", VERSION);
		printf("\n  -- MENU GENERAL --\n\n 1 - Pasar nuevo juego\n 2 - Ver listado de juegos en disco\n 3 - Borrar un juego\n 4 - Extraer un juego\n 5 - Crear canal HomeBrew\n 6 - Formatear disco para Wii\n 7 - Cambiar de unidad\n 0 - Salir\n\n");

		if (strlen(disk_id) != 0)								// muestra si se ha seleccionado unidad ya o no
			printf(" > Seleccionado disco: '%s'", disk_id);
		else
			printf(" > Aun no se ha seleccionado la unidad logica!");

		printf("\n\nInserta una opcion: ");						// pide opcion y recoge el valor
		scanf("%d", &opcion);
		
		if (opcion > 7 || opcion < 0)							// comprueba que la opcion sea valida
		{														// y si no, muestra mensaje de error
			while (getchar() != '\n');
			printf("\n  > Opcion incorrecta! Pulsa enter para continuar ...");
			getchar();
		}
	}
	while (getchar() != '\n');									// quita el intro al final del buffer de teclado

	return opcion;
}

void get_disk_id(char *disk)									// Muestra al usuario la lista de unidades logicas y pide escribir la correcta
{
	system("clear");											// limpia la pantalla
	
	printf("\n  -- Lista de unidades logicas --\n\n");			// muestra titulo
	chdir ("/dev");												// cambia a directorio /dev y muestra archivos disk*
	system("ls disk*");
	printf("\n\nIntroduce el identificador de unidad (ejemplo: disk3s1): ");
	
	scanf("%s", &(*disk));										// recoge identificador introducido
	getchar();													// espera al intro
}

int main (int argc, const char * argv[]) {						// funcion principal
    
	char	ruta[MAX_RUTA], comando_base[MAX_RUTA], comando[MAX_RUTA], filename[MAX_RUTA], disk[MAX_RUTA], game_id[10];
	int		i, opcion;
	FILE	*check;
	
	disk[0] = '\0';

	for (i=strlen(argv[0])-1; argv[0][i] != '/'; i--);			// obtiene la ruta actual quitando el nombre de la aplicacion del final
	
	strcpy(ruta, argv[0]);										// copia la ruta obtenida al registro "ruta"
	ruta[i] = '\0';
	
	strcpy(comando_base, "exec ");								// crea el comando base sin los argumentos
	strcat(comando_base, ruta);									// despues solo hay que hacer exec(comando);
	strcat(comando_base, "/wbfs_original");
	
	comando[0] = '\0';											// Comprueba que este el ejecutable en el mismo directorio
	strcpy(comando, ruta);										// para ello abre el archivo y comprueba que el puntero
	strcat(comando, "/wbfs_original");							// resultante no sea NULL

	check = fopen(comando,"rb");
	
	if (check == NULL)											// si no esta el archivo, muestra mensaje de error y rompe la ejecucion del
	{															// programa, yendo al final
		printf("\nError: no se encuentra el archivo 'wbfs_original' en este directorio!\n\n");
		goto fin;
	}
	else
	fclose(check);												// si el archivo existe, lo cierra y continua con lo demas

	opcion = menu(disk);
	
	while (opcion != SALIR)										// vuelve siempre al menu, hasta que la opcion sea 0, salir
	{
		switch(opcion)								
		{
			case ADD:
			{
				if (strlen(disk) == 0)							// comprueba si ya se ha identificado la unidad logica
					get_disk_id(disk);							// si aun no se ha identificado va a la funcion
				
				system("clear");								

				printf("\n  -- Pasar nuevo juego a disco --\n\nArrastra aqui el archivo .ISO  -->");
				
				fgets(filename, MAX_RUTA, stdin);				// lee ruta + nombre de archivo permitiendo espacios en la misma
				for ( i=0; filename[i] != '\n'; i++);			// despues limpia el intro al final de la cadena
					filename[i] = '\0';
				
				comando[0] = '\0';								// inicializa la cadena comando
				
				strcat(comando, comando_base);					// crea el comando a ejecutar concatenando las partes que necesita
				strcat(comando, " -d /dev/");
				strcat(comando, disk);
				strcat(comando, " add ");
				strcat(comando, filename);
				
				printf("\nPulsa intro para comenzar ...\n");	// espera al intro para dar la orden
				getchar();
				system(comando);
				printf("\nPulsa intro para volver al menu ...");// espera al intro para continuar
				getchar();
				
				break;
			}
		
			case LIST:
			{
				if (strlen(disk) == 0)							// comprueba si ya se ha identificado la unidad logica
					get_disk_id(disk);							// si aun no se ha identificado va a la funcion
				
				system("clear");

				comando[0] = '\0';								// inicializa la cadena comando

				strcat(comando, comando_base);					// crea el comando a ejecutar concatenando las partes que necesita
				strcat(comando, " -d /dev/");
				strcat(comando, disk);
				strcat(comando, " ls");
				
				printf("  -- Lista de juegos en disco --\n\n");
				
				system(comando);								// llama al ejecutable con las opciones añadidas ya
				printf("\nPulsa intro para volver al menu ... ");
				getchar();
				
				break;
			}
		
			case DEL:
			{
				if (strlen(disk) == 0)							// comprueba si ya se ha identificado la unidad logica
					get_disk_id(disk);							// si aun no se ha identificado va a la funcion
				
				system("clear");

				printf("\n  -- Borrar juego del disco --\n\n");
				
				comando[0] = '\0';

				strcat(comando, comando_base);
				strcat(comando, " -d /dev/");
				strcat(comando, disk);
				strcat(comando, " ls");
								
				system(comando);
				printf("\n\n > Introduce el Game-ID del juego a borrar: ");
				
				scanf("%s", game_id);
				
				comando[0] = '\0';

				strcat(comando, comando_base);
				strcat(comando, " -d /dev/");
				strcat(comando, disk);
				strcat(comando, " rm ");
				strcat(comando, game_id);
				
				while (getchar() != '\n');
				printf("\nPulsa enter para borrarlo ...");
				getchar();
				
				system(comando);

				
				printf("\nPulsa enter para volver al menu ...");
				getchar();
				
				break;
			}
		
			case EXTRACT:
			{
				if (strlen(disk) == 0)							// comprueba si ya se ha identificado la unidad logica
					get_disk_id(disk);							// si aun no se ha identificado va a la funcion
				
				system("clear");

				printf("\n  -- Extraer juego del disco --\n\n");
				
				comando[0] = '\0';

				strcat(comando, comando_base);
				strcat(comando, " -d /dev/");
				strcat(comando, disk);
				strcat(comando, " ls");
								
				system(comando);
				printf("\n\n > Introduce el Game-ID del juego a extraer: ");
				
				scanf("%s", game_id);
				
				comando[0] = '\0';

				strcat(comando, comando_base);
				strcat(comando, " -d /dev/");
				strcat(comando, disk);
				strcat(comando, " extract ");
				strcat(comando, game_id);
				
				chdir(ruta);									// cambia al directorio actual antes de hacer la llamada,
																// para que el archivo se genere en esa ubicacion
				while (getchar() != '\n');
				printf("\nPulsa enter para comenzar ...");
				getchar();
				
				system(comando);

				
				printf("\nPulsa enter para volver al menu ...");
				getchar();
			
				break;
			}
		
			case MAKEHBC:
			{
				if (strlen(disk) == 0)							// comprueba si ya se ha identificado la unidad logica
					get_disk_id(disk);							// si aun no se ha identificado va a la funcion
				
				system("clear");

				comando[0] = '\0';

				strcat(comando, comando_base);
				strcat(comando, " -d /dev/");
				strcat(comando, disk);
				strcat(comando, " mkhbc");
				
				chdir(ruta);									// cambia al directorio actual antes de hacer la llamada,
																// para que los archivos se generen en esa ubicacion
				
				printf("  -- Crear Homebrew Channel para juegos en disco USB --\n\nPulsa intro para continuar");
				getchar();
				
				system(comando);
				printf("\nPulsa intro para volver al menu ... ");
				getchar();
				
			
				break;
			}
		
			case FORMAT:
			{
				system("clear");
	
				printf("\n  -- Lista de unidades logicas --\n\n");
				chdir ("/dev");
				system("ls disk*");
				printf("\n\n¡¡ MUCHO CUIDADO AQUI !!\n\n > Introduce el identificador del disco a formatear: ");
	
				scanf("%s", disk);
				getchar();
				
				comando[0] = '\0';

				strcat(comando, comando_base);
				strcat(comando, " -p /dev/");
				strcat(comando, disk);
				strcat(comando, " init");
				
				printf("\nPulsa intro para comenzar el formato ...");
				getchar();
				system(comando);
				
				printf("\nPulsa intro para volver al menu ... ");
				getchar();
				
				break;
			}

			case CAMBIAR:
			{
				get_disk_id(disk);
				break;
			}
		}		
		opcion = menu(disk);
		
	}
	printf("\nAdios !\n\n");

	fin:														// etiqueta de salto al fin del programa

    return 0;
}
